#!/bin/bash
SCRIPT_DIR="$(cd "$(dirname "${BASH_SOURCE[0]}")" >/dev/null 2>&1 && pwd)"

cwd=$(pwd)

PROJECT_DIR_NAME=${cwd##*/}

source ${SCRIPT_DIR}/utilis/colors.sh
source ${SCRIPT_DIR}/functions.sh

function usage() {
  echo "${YELLOW}Description: ${RESET}"
  echo "    StarterKit Upgrader. "
  echo ""
  echo "${YELLOW}Usage: ${RESET}"
  echo "    ./bin/upgrade.sh [options]"
  echo ""
  echo "${YELLOW}Options: ${RESET}"
  echo "   ${GREEN} -h, --help     ${RESET} Display this help message"
  echo "   ${GREEN} -v, --version  ${RESET} The version of StarterKit to upgrade to"
}

function moveFilesToTemp() {
  local __path=${1}

  if [[ -z ${__path} ]]; then
    echo "Missing path"
    exit 1
  fi
  [[ ! -d "${cwd}/${__path}" ]] || rsync -a "${cwd}/${__path}/" "${TEMP_DIR}/${__path}"
}

# Read arguments
POSITIONAL=()
while [[ $# -gt 0 ]]; do
  key="$1"
  case $key in
  -h | --help)
    usage
    exit 0
    shift # past argument
    ;;
  -v | --version)
    VERSION=${2}
    shift # past argument
    shift # past argument
    ;;
  *) # unknown option
    POSITIONAL+=("$1") # save it in an array for later
    shift              # past argument
    ;;
  esac
done
set -- "${POSITIONAL[@]}" # restore positional parameters
# Done reading arguments

if [[ -z ${VERSION} ]]; then
  VERSIONS=($(getStarterKitVersions))
  if [[ $? != 0 ]]; then
    echo "Can't automatically find StarterKit version to upgrade to."
    echo "You'll have to find the available versions manually on \"https://composer.infomaker.io/#!/everyware-starter-kit\"."
    echo "Then run the upgrade with ' --version <version>'"
    exit 1
  fi
  VERSION=${VERSIONS[0]}
fi
echo "${GREEN}> Prepare upgrade of Starter Kit...${RESET}"

STARTER_KIT_ZIP="https://composer.infomaker.io/dist/infomaker-everyware-starter-kit-${VERSION}.zip"
TEMP_DIR="${cwd}/tmp/stkt-${VERSION}"

if [[ ! -d ${TEMP_DIR} ]]; then
  echo "${GREEN}> Downloading Starter Kit ${BOLD}v${VERSION}${RESET}"
  curl -L -o "${TEMP_DIR}.zip" --create-dirs ${STARTER_KIT_ZIP}
  unzip -q "${TEMP_DIR}.zip" -d ${TEMP_DIR}
fi

cd ${TEMP_DIR}

# Create a .env.example file
[[ ! -f .project-env ]] || [[ -f .env.example ]] || mv .project-env .env.example

# Create project .gitignore
[[ ! -f .project-gitignore ]] || [[ -f .gitignore ]] || mv .project-gitignore .gitignore

REMOVE_FILES=(
  mysql
  .editorconfig
  .eslintrc.js
  .sass-lint.yml
  gulpfile.js
  gulpfile-config.js
  package.json
  CHANGELOG.md
)

rm -rf ${REMOVE_FILES[@]}

cd ${cwd}

moveFilesToTemp "lua/utils"
moveFilesToTemp "nginx/app"

echo "${GREEN}>${BOLD} Preparations done!!${RESET} "
echo "> Proceed to update the project code...${RESET} "

# Compare bin-scripts to determine if some of them should be removed
echo "> Check for unknown files in ${SCRIPT_DIR}:${RESET} "

# Make sure that the new version has an upgrade script
[[ -f ${TEMP_DIR}/bin/upgrade.sh ]] || cp ${SCRIPT_DIR}/upgrade.sh ${TEMP_DIR}/bin/upgrade.sh

STKT_SCRIPTS=($(ls ${TEMP_DIR}/bin))

for file in "${SCRIPT_DIR}"/*; do
  filename=${file##*/}

  # If script-file dose not exists in new StarterKit version we offer to remove it
  if [[ ! " ${STKT_SCRIPTS[@]} " =~ " ${filename} " ]]; then
    echo ""
    echo "\"${filename}\" is not part of the new StarterKit version."
    read -p "Do you wish to remove it (y/n)? " -n 1 -r
    echo ""
    if [[ $REPLY =~ ^[Yy]$ ]]; then
      echo "Removing ${SCRIPT_DIR}/${filename}"
      rm -f ${SCRIPT_DIR}/${filename}
    fi
  fi
done

# Rename new upgrade file if it will change
if [[ $(diff bin/upgrade.sh ${SCRIPT_DIR}/upgrade.sh) ]]; then
  mv bin/upgrade.sh bin/upgrade-${VERSION}.sh
fi

REMOVE_FILES=(
  .ebextensions
  lua
  rockspec
  www/wp-config
)

rm -rf ${REMOVE_FILES[@]}

cd ${cwd}

rsync -a ${TEMP_DIR}/ .

# Clean Up
rm -rf ./tmp*

echo "${GREEN}>${BOLD} All done!!${RESET} "

# Check if any of the EB-config files has changed
git diff --exit-code ${cwd}/.ebextensions/ >>/dev/null
if [[ ${?} == 1 ]]; then
  echo "${BOLD}NOTE: ${RESET} "
  echo "========= "
  echo "Elastic Beanstalk config has changes with the new version of StarterKit. "
  echo "Further changes might have to be done in AWS in order for these changes to work. "
  echo
  git diff --exit-code ${cwd}/.ebextensions/
fi

if [[ -f ${SCRIPT_DIR}/upgrade-${VERSION}.sh ]]; then
  echo
  echo "${BOLD}NOTE: ${RESET} "
  echo "========= "
  echo "It seems the upgrade script has been changed with the new version of StarterKit. "
  echo "You need to manually update it. To do this run the following command: "
  echo
  echo "mv bin/upgrade-${VERSION}.sh bin/upgrade.sh "
  echo
fi
