#!/bin/bash
SCRIPT_DIR="$(cd "$(dirname "${BASH_SOURCE[0]}")" >/dev/null 2>&1 && pwd)"

cwd=$(pwd)

source ${SCRIPT_DIR}/utilis/colors.sh
source ${SCRIPT_DIR}/functions.sh

VERSION="4.9.15"
languages=("US" "DK" "NB" "NN" "SE")
WordpressDir="${cwd}/www"

FORCE=false
INTERACTION=true

function usage() {
  echo "${YELLOW}Description: ${RESET}"
  echo "    Install Wordpress. "
  echo ""
  echo "${YELLOW}Usage: ${RESET}"
  echo "    ./bin/wordpress-install.sh [options]"
  echo ""
  echo "${YELLOW}Options: ${RESET}"
  echo "   ${GREEN} -f, --force          ${RESET} Override existing installation"
  echo "   ${GREEN} -h, --help           ${RESET} Display this help message"
  echo "   ${GREEN} -l, --language       ${RESET} Specify a language for your Wordpress installation"
  echo "   ${GREEN} -n, --no-interaction ${RESET} Do not ask any interactive question"
  echo "   ${GREEN} -v, --version        ${RESET} The version of Wordpress to install"
}

# Read arguments
POSITIONAL=()
while [[ $# -gt 0 ]]; do
  key="$1"
  case $key in
  -f | --force)
    FORCE=true
    shift # past argument
    ;;
  -h | --help)
    usage
    exit 0
    shift # past argument
    ;;
  -l | --language)
    LANGUAGE=$(echo ${2} | awk '{print toupper($0)}')
    shift # past argument
    shift # past argument
    ;;
  -n | --no-interaction)
    INTERACTION=false
    shift # past argument
    ;;
  -v | --version)
    VERSION=${2}
    shift # past argument
    shift # past argument
    ;;
  *) # unknown option
    POSITIONAL+=("$1") # save it in an array for later
    shift              # past argument
    ;;
  esac
done
set -- "${POSITIONAL[@]}" # restore positional parameters
# Done reading arguments

# If not running "force", offer way to abort if Wordpress installation exists
if [[ -f "${WordpressDir}/wp-blog-header.php" ]] && [[ ${FORCE} != true ]]; then

  if [[ ${INTERACTION} == true ]]; then
    echo ""
    echo "${BOLD}[INFO]${RESET} Wordpress is already installed."
    read -p "Do you want to replace existing Wordpress installation y/n ? " -n 1 -r
    echo ""
    if [[ ! $REPLY =~ ^[Yy]$ ]]; then
      echo "${BOLD}[INFO]${RESET} Nothing was been installed. Have a nice day!${RESET}"
      exit 0
    fi
  else
    echo "${BOLD}[INFO]${RESET} Wordpress already exists. Nothing was installed.${RESET}"
    exit 0
  fi
fi

# If language when not provided
if [[ -z ${LANGUAGE} ]]; then

  # Prompt for language when not provided
  if [[ ${FORCE} != true ]] && [[ ${INTERACTION} == true ]]; then
    echo "${GREEN}${BOLD}> Language of WordPress installation (1-${#languages[@]})?${RESET}"
    PS3="Option?"
    select wp_language in ${languages[@]}; do
      LANGUAGE=${wp_language}
      break
    done
  else
    LANGUAGE=US
  fi
fi

# Remove existing installation before installing new one
if [[ -f "${WordpressDir}/wp-blog-header.php" ]]; then
  ${SCRIPT_DIR}/wordpress-remove.sh --force
fi

installWordpress ${VERSION} ${LANGUAGE} ${WordpressDir}

# Check if Wordpress was installed
if [[ ! -f "${WordpressDir}/wp-blog-header.php" ]]; then
  echo "${RED}${BOLD}> Oops! Something went wrong with the Wordpress installation ${RESET}"
  exit 1
fi

${SCRIPT_DIR}/wordpress-update-salt.sh
echo ""
echo "${GREEN}${BOLD}> DONE!${RESET}"
